﻿Imports System
Imports System.Threading
Imports System.IO.Ports

Public Class Form1

    ' Command ******************************************************************
    Const CMD_ReadData As Byte = &H81                           ' 命令: 讀取資料
    Const CMD_L1_Set As Byte = &H91                             ' 命令: L1設定值
    Const CMD_L2_Set As Byte = &H92                             ' 命令: L2設定值
    Const CMD_L1_Output As Byte = &HA1                          ' 命令: L1輸出值
    Const CMD_L2_Output As Byte = &HA2                          ' 命令: L2輸出值
    Const CMD_L1_OnOff As Byte = &HC1                           ' 命令: L1 ON/OFF
    Const CMD_L2_OnOff As Byte = &HC2                           ' 命令: L2 ON/OFF
    Const CMD_L1_TrigOut As Byte = &HD1                         ' 命令: L1觸發輸出延遲
    Const CMD_L2_TrigOut As Byte = &HD2                         ' 命令: L2觸發輸出延遲

    ' Lx_Set ===================================================================
    '                                                             bit7~4 最大電流
    Const SET_ExtOnOff As Byte = &H2                            ' bit1   外部ON/OFF: 0=除能, 1=致能
    Const SET_OnOff As Byte = &H1                               ' bit0   ON/OFF    : 0=OFF , 1=ON

    ' ACK ======================================================================
    Const ACK_Failed As Byte = &H80                             ' bit7 命令執行狀態: 0=成功, 1=失敗
    Const ACK_LowVolt As Byte = &H10                            ' bit4 電源低電壓  : 0=No,   1=Yes
    Const ACK_L2_OC As Byte = &H2                               ' bit1 L2過流      : 0=No,   1=Yes
    Const ACK_L1_OC As Byte = &H1                               ' bit0 L1過流      : 0=No,   1=Yes

    ' Baud Rate ================================================================
    Const BaudRate_LS As Byte = 0                               ' 9600 8-N-1
    Const BaudRate_MS As Byte = 1                               ' 38400 8-N-1
    Const BaudRate_HS As Byte = 2                               ' 115200 8-N-1

    ' ==========================================================================
    Const LineMax As UShort = 256                               ' 最大行數

    ' **************************************************************************
    Dim LineNo As UShort = 0                                    ' 行數編號
    Dim LineCount As UShort = 0                                 ' 行數計數
    ' --------------------------------------------------------------------------
    Dim PortNo() As String                                      ' 埠編號
    Dim FG_ComPort As Boolean = False                           ' 旗標-開啟串列埠
    Dim FG_ReadData As Boolean = False                          ' 旗標-讀取資料
    ' --------------------------------------------------------------------------
    Dim CNT_AckTimeOut As Byte = 0                              ' 計時-回應逾時
    Dim CNT_AskAuto As Byte = 0                                 ' 計時-自動詢問
    Dim CNT_Flash As Byte = 0                                   ' 計時-閃爍
    ' 最大電流 -----------------------------------------------------------------
    Dim MaxCurrent() As Integer = {50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800}

    ' TX =======================================================================
    Dim FG_Transmit As Boolean = False                          ' 旗標-傳送
    Dim TX_Packet(7) As Byte                                    ' 傳送-封包
    Dim TX_Command As Byte = 0                                  ' 傳送-命令
    Dim TX_Count As Byte = 0                                    ' 傳送-計次

    ' RX =======================================================================
    Dim FG_Receive As Boolean = False                           ' 旗標-接收
    Dim RX_Packet(13) As Byte                                   ' 接收-封包
    Dim RX_Index As Byte = 0                                    ' 接收-封包索引
    Dim RX_Length As Byte = 0                                   ' 接收-封包長度
    Dim ACK As Byte = 0                                         ' 回應碼

    ' L1 =======================================================================
    Dim CHG_L1_Set As Boolean = False                           ' 變更-L1設定值
    Dim CHG_L1_Output As Boolean = False                        ' 變更-L1輸出值
    Dim CHG_L1_OnOff As Boolean = False                         ' 變更-L1 ON/OFF
    Dim CHG_L1_TrigOut As Boolean = False                       ' 變更-L1觸發輸出延遲
    Dim L1_Set As Byte = 0                                      ' L1設定值
    Dim L1_Output As Integer = 0                                ' L1輸出值
    Dim L1_TrigOut As Byte = 0                                  ' L1觸發輸出延遲

    ' L2 =======================================================================
    Dim CHG_L2_Set As Boolean = False                           ' 變更-L2設定值
    Dim CHG_L2_Output As Boolean = False                        ' 變更-L2輸出值
    Dim CHG_L2_OnOff As Boolean = False                         ' 變更-L2 ON/OFF
    Dim CHG_L2_TrigOut As Boolean = False                       ' 變更-L2觸發輸出延遲
    Dim L2_Set As Byte = 0                                      ' L2設定值
    Dim L2_Output As Integer = 0                                ' L2輸出值
    Dim L2_TrigOut As Byte = 0                                  ' L2觸發輸出延遲

    ' **************************************************************************

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        cbb_L1_MaxC.SelectedIndex = 0                           ' L1最大電流 = 50mA
        lb_L1_MaxC.Text = Convert.ToString(MaxCurrent(0))

        cbb_L2_MaxC.SelectedIndex = 0                           ' L2最大電流 = 50mA
        lb_L2_MaxC.Text = Convert.ToString(MaxCurrent(0))

        cbb_BaudRate.SelectedIndex = BaudRate_HS                ' 115200 8-N-1
        ComPort_Detect()                                        ' 串列埠檢知
    End Sub

    Private Sub timer_Tick(sender As Object, e As EventArgs) Handles timer.Tick
        If FG_ComPort = True Then                               ' 串列埠已開啟?
            If FG_Receive = False Then                          ' 尚無接收有效封包?
                If FG_Transmit = False Then                     ' 尚無傳送命令?
                    Check_TX_Command()                          ' 檢查-傳送命令
                Else
                    Check_RX_TimeOut()                          ' 檢查-接收逾時
                End If
                Ask_Auto()                                      ' 自動詢問
            Else                                                ' 已完成接收封包
                FG_Receive = False
                CNT_AckTimeOut = 0
                FG_Transmit = False
                TX_Count = 0
                If FG_ReadData = False Then                     ' 未讀取過資料?
                    Dsp_RX_Packet(Color.White, 13)
                    Update_L1()                                 ' 更新-L1參數
                    Update_L2()                                 ' 更新-L2參數
                    Check_ACK()                                 ' 檢查-回應碼
                    FG_ReadData = True
                Else                                            '  已讀取過資料
                    Dsp_RX_Packet(Color.White, 5)
                    Check_ACK()                                 ' 檢查-回應碼
                End If
            End If

            If (ACK And &H13) <> 0 Then                         ' 電源低電壓/L2過流/L1過流 ?
                Alarm_Flash()                                   ' 警告訊息-閃爍 (2Hz)
            End If
        End If
    End Sub

    ' ==========================================================================

    Private Sub Check_TX_Command()                              ' 檢查-傳送命令
        If CHG_L1_Set = True Then                               ' 變更-L1設定值?
            TX_L1_Set()                                         ' 傳送-L1設定值
            Return
        End If

        If CHG_L2_Set = True Then                               ' 變更-L2設定值?
            TX_L2_Set()                                         ' 傳送-L2設定值
            Return
        End If

        If CHG_L1_Output = True Then                            ' 變更-L1輸出值?
            TX_L1_Output()                                      ' 傳送-L1輸出值
            Return
        End If

        If CHG_L2_Output = True Then                            ' 變更-L2輸出值?
            TX_L2_Output()                                      ' 傳送-L2輸出值
            Return
        End If

        If CHG_L1_OnOff = True Then                             ' 變更-L1 ON/OFF ?
            TX_L1_OnOff()                                       ' 傳送-L1 ON/OFF
            Return
        End If

        If CHG_L2_OnOff = True Then                             ' 變更-L2 ON/OFF ?
            TX_L2_OnOff()                                       ' 傳送-L2 ON/OFF
            Return
        End If

        If CHG_L1_TrigOut = True Then                           ' 變更-L1觸發輸出延遲?
            TX_L1_TrigOut()                                     ' 傳送-L1觸發輸出延遲
            Return
        End If

        If CHG_L2_TrigOut = True Then                           ' 變更-L2觸發輸出延遲?
            TX_L2_TrigOut()                                     ' 傳送-L2觸發輸出延遲
            Return
        End If
    End Sub

    Private Sub Check_RX_TimeOut()                              ' 檢查-接收逾時
        CNT_AckTimeOut += 1
        If CNT_AckTimeOut >= 10 Then                            ' 回應逾時? (10 * 50ms = 500ms)
            CNT_AckTimeOut = 0
            If TX_Count > 2 Then
                Dsp_String(Color.Red, " 逾時未回應!")
                ComPort_Disconnect()                            ' 串列埠離線
            Else
                Select Case TX_Command
                    Case CMD_ReadData
                        TX_ReadData()                           ' 傳送-讀取資料
                    Case CMD_L1_Set
                        TX_L1_Set()                             ' 傳送-L1設定值
                    Case CMD_L2_Set
                        TX_L2_Set()                             ' 傳送-L2設定值
                    Case CMD_L1_Output
                        TX_L1_Output()                          ' 傳送-L1輸出值
                    Case CMD_L2_Output
                        TX_L2_Output()                          ' 傳送-L2輸出值
                    Case CMD_L1_OnOff
                        TX_L1_OnOff()                           ' 傳送-L1 ON/OFF
                    Case CMD_L2_OnOff
                        TX_L2_OnOff()                           ' 傳送-L2 ON/OFF
                    Case CMD_L1_TrigOut
                        TX_L1_TrigOut()                         ' 傳送-L1觸發輸出延遲
                    Case CMD_L2_TrigOut
                        TX_L2_TrigOut()                         ' 傳送-L2觸發輸出延遲
                End Select
            End If
        End If
    End Sub

    Private Sub Ask_Auto()                                      ' 自動詢問
        If cbb_L1_MaxC.DroppedDown = True Then
            CNT_AskAuto = 0
            Return
        End If

        If cbb_L2_MaxC.DroppedDown = True Then
            CNT_AskAuto = 0
            Return
        End If

        If nud_L1_Output.Focused = True Then
            CNT_AskAuto = 0
            Return
        End If

        If nud_L2_Output.Focused = True Then
            CNT_AskAuto = 0
            Return
        End If

        If nud_L1_TrigOut.Focused = True Then
            CNT_AskAuto = 0
            Return
        End If

        If nud_L2_TrigOut.Focused = True Then
            CNT_AskAuto = 0
            Return
        End If

        CNT_AskAuto += 1
        If CNT_AskAuto >= 60 Then                               ' 自動詢問? (60 * 50ms = 3sec)
            TX_ReadData()                                       ' 傳送-讀取資料
        End If
    End Sub

    Private Sub Update_L1()                                     ' 更新-L1參數
        L1_Set = RX_Packet(4) And &HF3

        If (L1_Set And SET_OnOff) = 0 Then                      ' L1 OFF ?
            lb_L1_LED.BackColor = Color.DimGray
            btn_L1_OnOff.Text = "ON"
        Else                                                    ' L1 ON
            lb_L1_LED.BackColor = Color.Green
            btn_L1_OnOff.Text = "OFF"
        End If

        If (L1_Set And SET_ExtOnOff) = 0 Then                   ' L1外部ON/OFF-除能?
            ckb_L1_ExtOnOff.Checked = False
            lb_L1_LED.Visible = True
            btn_L1_OnOff.Visible = True
        Else                                                    ' L1外部ON/OFF-致能
            ckb_L1_ExtOnOff.Checked = True
            lb_L1_LED.Visible = False
            btn_L1_OnOff.Visible = False
        End If

        ' L1最大電流 -----------------------------------------------------------
        cbb_L1_MaxC.SelectedIndex = (L1_Set >> 4) And &HF
        Dim MaxC As Integer = MaxCurrent(cbb_L1_MaxC.SelectedIndex)

        lb_L1_MaxC.Text = Convert.ToString(MaxC)
        tkb_L1_Output.TickFrequency = MaxC / 10
        nud_L1_Output.Maximum = MaxC
        tkb_L1_Output.Maximum = MaxC

        ' L1輸出值 -------------------------------------------------------------
        Dim Output As Integer = RX_Packet(5)
        Output <<= 8
        Output = Output Or RX_Packet(6)

        nud_L1_Output.Value = Output
        tkb_L1_Output.Value = Output
        L1_Output = Output
        lb_L1_PCV.Text = String.Format("{0:F1}%", (Output * 100) / MaxC)

        ' L1觸發輸出延遲 -------------------------------------------------------
        L1_TrigOut = RX_Packet(7)
        nud_L1_TrigOut.Value = L1_TrigOut
        lb_L1_TrigOut.Text = Convert.ToString(L1_TrigOut * 5) + "ms"
    End Sub

    Private Sub Update_L2()                                     ' 更新-L2參數
        L2_Set = RX_Packet(8) And &HF3

        If (L2_Set And SET_OnOff) = 0 Then                      ' L2 OFF ?
            lb_L2_LED.BackColor = Color.DimGray
            btn_L2_OnOff.Text = "ON"
        Else                                                    ' L2 ON
            lb_L2_LED.BackColor = Color.Green
            btn_L2_OnOff.Text = "OFF"
        End If

        If (L2_Set And SET_ExtOnOff) = 0 Then                   ' L2外部ON/OFF-除能?
            ckb_L2_ExtOnOff.Checked = False
            lb_L2_LED.Visible = True
            btn_L2_OnOff.Visible = True
        Else                                                    ' L2外部ON/OFF-致能
            ckb_L2_ExtOnOff.Checked = True
            lb_L2_LED.Visible = False
            btn_L2_OnOff.Visible = False
        End If

        ' L2最大電流 -----------------------------------------------------------
        cbb_L2_MaxC.SelectedIndex = (L2_Set >> 4) And &HF
        Dim MaxC As Integer = MaxCurrent(cbb_L2_MaxC.SelectedIndex)

        lb_L2_MaxC.Text = Convert.ToString(MaxC)
        tkb_L2_Output.TickFrequency = MaxC / 10
        nud_L2_Output.Maximum = MaxC
        tkb_L2_Output.Maximum = MaxC

        ' L2輸出值 -------------------------------------------------------------
        Dim Output As Integer = RX_Packet(9)
        Output <<= 8
        Output = Output Or RX_Packet(10)

        nud_L2_Output.Value = Output
        tkb_L2_Output.Value = Output
        L2_Output = Output
        lb_L2_PCV.Text = String.Format("{0:F1}%", (Output * 100) / MaxC)

        ' L2觸發輸出延遲 -------------------------------------------------------
        L2_TrigOut = RX_Packet(11)
        nud_L2_TrigOut.Value = L2_TrigOut
        lb_L2_TrigOut.Text = Convert.ToString(L2_TrigOut * 5) + "ms"
    End Sub

    Private Sub Check_ACK()                                     ' 檢查-回應碼
        ACK = RX_Packet(3) And &H93

        If (ACK And ACK_Failed) = ACK_Failed Then               ' bit7: 命令執行失敗?
            Dsp_String(Color.Red, " 命令執行失敗!")
        End If

        If (ACK And ACK_LowVolt) = ACK_LowVolt Then             ' bit4: 電源低電壓?
            Dsp_String(Color.Red, " 電源低電壓!")
        Else
            lb_LowVolt.BackColor = Color.DimGray
        End If

        If (ACK And &H3) = &H3 Then                             ' L2過流 & L1過流 ?
            Dsp_String(Color.Red, " L1-L2過流!")
        Else
            If (ACK And ACK_L2_OC) = ACK_L2_OC Then             ' bit1: L2過流?
                Dsp_String(Color.Red, " L2過流!")
            Else
                lb_L2_OC.BackColor = Color.DimGray
            End If

            If (ACK And ACK_L1_OC) = ACK_L1_OC Then             ' bit0: L1過流?
                Dsp_String(Color.Red, " L1過流!")
            Else
                lb_L1_OC.BackColor = Color.DimGray
            End If
        End If

        If (ACK And &H13) <> 0 Then                             ' 電源低電壓/L2過流/L1過流 ?
            Lx_GroupBox_Enabled(False)                          ' Lx群組方塊 (不啟用)
        Else
            Lx_GroupBox_Enabled(True)                           ' Lx群組方塊 (啟用)
        End If
    End Sub

    Private Sub Alarm_Flash()                                   ' 警告訊息-閃爍 (2Hz)
        If (ACK And ACK_LowVolt) = ACK_LowVolt Then             ' bit4: 電源低電壓?
            If CNT_Flash = 0 Then
                lb_LowVolt.BackColor = Color.Red
            End If

            If CNT_Flash = 5 Then
                lb_LowVolt.BackColor = Color.DimGray
            End If
        End If

        If (ACK And ACK_L2_OC) = ACK_L2_OC Then                 ' bit1: L2過流?
            If CNT_Flash = 0 Then
                lb_L2_OC.BackColor = Color.Red
            End If

            If CNT_Flash = 5 Then
                lb_L2_OC.BackColor = Color.DimGray
            End If
        End If

        If (ACK And ACK_L1_OC) = ACK_L1_OC Then                 ' bit0: L1過流?
            If CNT_Flash = 0 Then
                lb_L1_OC.BackColor = Color.Red
            End If

            If CNT_Flash = 5 Then
                lb_L1_OC.BackColor = Color.DimGray
            End If
        End If

        CNT_Flash += 1
        If CNT_Flash > 9 Then                                   ' 10 * 50ms = 500ms
            CNT_Flash = 0
        End If
    End Sub

    ' RichTextBox 顯示 *********************************************************

    Private Sub Dsp_String(ByVal text_color As Color, ByVal str As String)  ' 顯示字串
        LineCount += 1
        If LineCount > LineMax Then                             ' 行數計數 > 最大行數?
            LineCount = 1
            richTextBox.Clear()
        End If

        If LineCount > 1 Then
            richTextBox.AppendText(vbCrLf)
        End If
        richTextBox.SelectionStart = richTextBox.Text.Length
        richTextBox.ScrollToCaret()

        ' 行數編號顯示 ---------------------------------------------------------
        richTextBox.SelectionBackColor = Color.DimGray
        richTextBox.SelectionColor = Color.White
        richTextBox.AppendText(String.Format("{0:X4}", LineNo))
        LineNo += 1

        ' 字串顯示 -------------------------------------------------------------
        richTextBox.SelectionBackColor = Color.Black
        richTextBox.SelectionColor = text_color
        richTextBox.AppendText(str)
        richTextBox.Select()
    End Sub

    Private Sub Dsp_TX_Packet(ByVal text_color As Color, ByVal length As Byte)  ' 顯示 TX 封包
        LineCount += 1
        If LineCount > LineMax Then                             ' 行數計數 > 最大行數?
            LineCount = 1
            richTextBox.Clear()
        End If

        If LineCount > 1 Then
            richTextBox.AppendText(vbCrLf)
        End If
        richTextBox.SelectionStart = richTextBox.Text.Length
        richTextBox.ScrollToCaret()

        ' 行數編號顯示 ---------------------------------------------------------
        richTextBox.SelectionBackColor = Color.DimGray
        richTextBox.SelectionColor = Color.White
        richTextBox.AppendText(String.Format("{0:X4}", LineNo))
        LineNo += 1

        ' TX 封包 -> 字串 ------------------------------------------------------
        Dim str As String = " TX:"
        Dim i As Byte
        For i = 0 To (length - 1)
            str += String.Format(" {0:X2}", TX_Packet(i))
        Next
        str += " (" + Convert.ToString(TX_Count) + ")"

        ' 字串顯示 -------------------------------------------------------------
        richTextBox.SelectionBackColor = Color.Black
        richTextBox.SelectionColor = text_color
        richTextBox.AppendText(str)
        richTextBox.Select()
    End Sub

    Private Sub Dsp_RX_Packet(ByVal text_color As Color, ByVal length As Byte)  ' 顯示 RX 封包
        LineCount += 1
        If LineCount > LineMax Then                             ' 行數計數 > 最大行數?
            LineCount = 1
            richTextBox.Clear()
        End If

        If LineCount > 1 Then
            richTextBox.AppendText(vbCrLf)
        End If
        richTextBox.SelectionStart = richTextBox.Text.Length
        richTextBox.ScrollToCaret()

        ' 行數編號顯示 ---------------------------------------------------------
        richTextBox.SelectionBackColor = Color.DimGray
        richTextBox.SelectionColor = Color.White
        richTextBox.AppendText(String.Format("{0:X4}", LineNo))
        LineNo += 1

        ' RX 封包 -> 字串 ------------------------------------------------------
        Dim str As String = " RX:"
        Dim i As Byte
        For i = 0 To (length - 1)
            str += String.Format(" {0:X2}", RX_Packet(i))
        Next

        ' 字串顯示 -------------------------------------------------------------
        richTextBox.SelectionBackColor = Color.Black
        richTextBox.SelectionColor = text_color
        richTextBox.AppendText(str)
        richTextBox.Select()
    End Sub

    ' 串列埠 *******************************************************************

    Private Sub cbb_ComPort_Click(sender As Object, e As EventArgs) Handles cbb_ComPort.Click
        ComPort_Detect()                                        ' 串列埠檢知
    End Sub

    Private Sub btn_Connect_Click(sender As Object, e As EventArgs) Handles btn_Connect.Click
        Dim state As Boolean = False

        If serialPort.IsOpen Then                               ' 串列埠已開啟?
            ComPort_Disconnect()                                ' 串列埠離線
        Else                                                    ' 串列埠未開啟
            PortNo = SerialPort.GetPortNames()
            If PortNo.Length >= 1 Then
                If cbb_ComPort.Text = "" Then
                    MessageBox.Show("請選擇有效存在的串列埠.", "未選擇串列埠", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Else
                    For Each port As String In PortNo
                        If port = cbb_ComPort.Text Then
                            state = True
                            Exit For
                        End If
                    Next port

                    If state = True Then
                        If serialPort.IsOpen Then               ' 串列埠已開啟?
                            ComPort_Disconnect()                ' 串列埠離線
                        Else                                    ' 串列埠未開啟
                            ComPort_Connect()                   ' 串列埠連線
                            If FG_ComPort = True Then           ' 串列埠已連線?
                                TX_ReadData()                   ' 傳送-讀取資料
                            End If
                        End If
                    End If
                End If
            Else
                cbb_ComPort.Items.Clear()
                MessageBox.Show("請檢視串列埠.", "未發現任何串列埠", MessageBoxButtons.OK, MessageBoxIcon.Information)
            End If
        End If
    End Sub

    Private Sub ComPort_Detect()                                ' 串列埠檢知
        Dim Port_No As String = cbb_ComPort.Text
        cbb_ComPort.Text = ""
        cbb_ComPort.Items.Clear()

        PortNo = SerialPort.GetPortNames()                      ' 取得存在的串列埠
        If PortNo.Length >= 1 Then
            For Each port As String In PortNo                   ' 將找到的現有串列埠加入下拉式方塊中的項目
                cbb_ComPort.Items.Add(port)
                If port = Port_No Then
                    cbb_ComPort.Text = Port_No
                End If
            Next port

            If cbb_ComPort.Text = "" Then
                cbb_ComPort.Text = PortNo(0)                    ' 下拉式方塊先顯示一個現存的串列埠
            End If
        Else
            MessageBox.Show("請檢視串列埠.", "未發現任何串列埠", MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
    End Sub

    Private Sub ComPort_Connect()                               ' 串列埠連線
        ' 設定串列埠鮑率 -------------------------------------------------------
        serialPort.PortName = cbb_ComPort.Text
        Select Case cbb_BaudRate.SelectedIndex
            Case BaudRate_LS                                    ' 9600 8-N-1
                serialPort.BaudRate = 9600
            Case BaudRate_MS                                    ' 38400 8-N-1
                serialPort.BaudRate = 38400
            Case BaudRate_HS                                    ' 115200 8-N-1
                serialPort.BaudRate = 115200
        End Select

        Try
            serialPort.Open()
            serialPort.DiscardInBuffer()
            serialPort.DiscardOutBuffer()

            cbb_BaudRate.Enabled = False
            cbb_ComPort.Enabled = False

            btn_Connect.Text = "Disconnect"
            tss_Status.Text = "Connect"
            FG_ComPort = True
        Catch ex As Exception
            MessageBox.Show("串列埠開啟程序發生不可預期錯誤!\r串列埠開啟失敗.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub ComPort_Disconnect()                            ' 串列埠離線
        FG_ComPort = False
        FG_Receive = False
        FG_Transmit = False
        FG_ReadData = False

        ACK = 0
        RX_Index = 0
        TX_Command = 0
        TX_Count = 0

        CHG_L1_Set = False
        CHG_L1_Output = False
        CHG_L1_OnOff = False
        CHG_L1_TrigOut = False

        CHG_L2_Set = False
        CHG_L2_Output = False
        CHG_L2_OnOff = False
        CHG_L2_TrigOut = False

        Try
            serialPort.Close()
        Catch ex As Exception
            ComPort_Detect()                                    ' 串列埠檢知
        End Try

        Lx_GroupBox_Enabled(False)                              ' Lx群組方塊 (不啟用)
        lb_L1_LED.BackColor = Color.DimGray
        lb_L2_LED.BackColor = Color.DimGray

        lb_L1_OC.BackColor = Color.DimGray
        lb_L2_OC.BackColor = Color.DimGray
        lb_LowVolt.BackColor = Color.DimGray

        cbb_BaudRate.Enabled = True
        cbb_ComPort.Enabled = True

        btn_Connect.Text = "Connect"
        tss_Status.Text = "Disconnect"
    End Sub

    Private Sub Lx_GroupBox_Enabled(ByVal s As Boolean)         ' Lx群組方塊啟用否?
        gpb_L1_OnOffCtrl.Enabled = s
        gpb_L1_Output.Enabled = s
        gpb_L1_TrigOut.Enabled = s

        gpb_L2_OnOffCtrl.Enabled = s
        gpb_L2_Output.Enabled = s
        gpb_L2_TrigOut.Enabled = s
    End Sub

    ' 串列埠傳送 ***************************************************************

    Private Sub TX_ReadData()                                   ' 傳送-讀取資料
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 5                                    ' 封包長度
            TX_Packet(3) = CMD_ReadData                         ' 命令碼

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 3
                checksum += TX_Packet(i)
            Next
            TX_Packet(4) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_ReadData
            TX_Count = 0
            FG_Transmit = True
            FG_ReadData = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.White, 5)
        Else
            Dsp_TX_Packet(Color.Magenta, 5)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 5)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-讀取資料] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub TX_L1_Set()                                     ' 傳送-L1設定值
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 6                                    ' 封包長度
            TX_Packet(3) = CMD_L1_Set                           ' 命令碼
            TX_Packet(4) = L1_Set                               ' L1設定值

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 4
                checksum += TX_Packet(i)
            Next
            TX_Packet(5) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_L1_Set
            TX_Count = 0
            FG_Transmit = True
            CHG_L1_Set = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.Yellow, 6)
        Else
            Dsp_TX_Packet(Color.Magenta, 6)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 6)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-L1設定值] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub TX_L2_Set()                                     ' 傳送-L2設定值
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 6                                    ' 封包長度
            TX_Packet(3) = CMD_L2_Set                           ' 命令碼
            TX_Packet(4) = L2_Set                               ' L2設定值

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 4
                checksum += TX_Packet(i)
            Next
            TX_Packet(5) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_L2_Set
            TX_Count = 0
            FG_Transmit = True
            CHG_L2_Set = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.Yellow, 6)
        Else
            Dsp_TX_Packet(Color.Magenta, 6)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 6)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-L2設定值] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub TX_L1_Output()                                  ' 傳送-L1輸出值
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 7                                    ' 封包長度
            TX_Packet(3) = CMD_L1_Output                        ' 命令碼
            TX_Packet(4) = L1_Output >> 8                       ' L1輸出值 (HB)
            TX_Packet(5) = L1_Output And 255                    ' L1輸出值 (LB)

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 5
                checksum += TX_Packet(i)
            Next
            TX_Packet(6) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_L1_Output
            TX_Count = 0
            FG_Transmit = True
            CHG_L1_Output = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.Cyan, 7)
        Else
            Dsp_TX_Packet(Color.Magenta, 7)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 7)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-L1輸出值] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub TX_L2_Output()                                  ' 傳送-L2輸出值
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 7                                    ' 封包長度
            TX_Packet(3) = CMD_L2_Output                        ' 命令碼
            TX_Packet(4) = L2_Output >> 8                       ' L2輸出值, HB
            TX_Packet(5) = L2_Output And 255                    ' L2輸出值, LB

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 5
                checksum += TX_Packet(i)
            Next
            TX_Packet(6) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_L2_Output
            TX_Count = 0
            FG_Transmit = True
            CHG_L2_Output = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.Cyan, 7)
        Else
            Dsp_TX_Packet(Color.Magenta, 7)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 7)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-L2輸出值] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub TX_L1_OnOff()                                   ' 傳送-L1 ON/OFF
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 6                                    ' 封包長度
            TX_Packet(3) = CMD_L1_OnOff                         ' 命令碼
            TX_Packet(4) = L1_Set And SET_OnOff                 ' L! ON/OFF

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 4
                checksum += TX_Packet(i)
            Next
            TX_Packet(5) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_L1_OnOff
            TX_Count = 0
            FG_Transmit = True
            CHG_L1_OnOff = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.Lime, 6)
        Else
            Dsp_TX_Packet(Color.Magenta, 6)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 6)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-L1 ON/OFF] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub TX_L2_OnOff()                                   ' 傳送-L2 ON/OFF
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 6                                    ' 封包長度
            TX_Packet(3) = CMD_L2_OnOff                         ' 命令碼
            TX_Packet(4) = L2_Set And SET_OnOff                 ' L2 ON/OFF

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 4
                checksum += TX_Packet(i)
            Next
            TX_Packet(5) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_L2_OnOff
            TX_Count = 0
            FG_Transmit = True
            CHG_L2_OnOff = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.Lime, 6)
        Else
            Dsp_TX_Packet(Color.Magenta, 6)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 6)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-L2 ON/OFF] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub TX_L1_TrigOut()                                 ' 傳送-L1觸發輸出延遲
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 6                                    ' 封包長度
            TX_Packet(3) = CMD_L1_TrigOut                       ' 命令碼
            TX_Packet(4) = L1_TrigOut                           ' L1觸發輸出延遲

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 4
                checksum += TX_Packet(i)
            Next
            TX_Packet(5) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_L1_TrigOut
            TX_Count = 0
            FG_Transmit = True
            CHG_L1_TrigOut = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.Pink, 6)
        Else
            Dsp_TX_Packet(Color.Magenta, 6)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 6)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-L1觸發輸出延遲] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    Private Sub TX_L2_TrigOut()                                 ' 傳送-L2觸發輸出延遲
        If FG_Transmit = False Then
            TX_Packet(0) = &H4D                                 ' Header 'M'
            TX_Packet(1) = &H53                                 ' Header 'S'
            TX_Packet(2) = 6                                    ' 封包長度
            TX_Packet(3) = CMD_L2_TrigOut                       ' 命令碼
            TX_Packet(4) = L2_TrigOut                           ' L1觸發輸出延遲

            Dim checksum As UShort = 0
            Dim i As Byte
            For i = 0 To 4
                checksum += TX_Packet(i)
            Next
            TX_Packet(5) = checksum And 255                     ' 檢查碼

            TX_Command = CMD_L2_TrigOut
            TX_Count = 0
            FG_Transmit = True
            CHG_L2_TrigOut = False
        End If

        CNT_AckTimeOut = 0
        CNT_AskAuto = 0
        TX_Count += 1
        If TX_Count = 1 Then
            Dsp_TX_Packet(Color.Pink, 6)
        Else
            Dsp_TX_Packet(Color.Magenta, 6)
        End If

        Try
            serialPort.Write(TX_Packet, 0, 6)
        Catch ex As Exception
            ComPort_Disconnect()                                ' 串列埠離線
            MessageBox.Show("[傳送-L2觸發輸出延遲] 發生不可預期錯誤!\r串列埠已關閉.", "例外狀況", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ' 串列埠接收 ***************************************************************

    Private Sub serialPort_DataReceived(sender As Object, e As SerialDataReceivedEventArgs) Handles serialPort.DataReceived
        Dim bytes As Integer = serialPort.BytesToRead

        If bytes > 0 Then
            Dim buffer(bytes) As Byte
            serialPort.Read(buffer, 0, bytes)

            Dim i As Integer
            For i = 0 To (bytes - 1)
                RX_Packet(RX_Index) = buffer(i)
                If RX_Index = 0 Then                            ' Header 'S'
                    If RX_Packet(0) = &H53 Then
                        RX_Index += 1
                    End If
                Else
                    If RX_Index = 1 Then                        ' Header 'M'
                        If RX_Packet(1) = &H4D Then
                            RX_Index += 1
                        Else
                            RX_Index = 0
                        End If
                    Else
                        If RX_Index = 2 Then                    ' 封包長度
                            RX_Length = RX_Packet(2)
                            If (RX_Length = 5) Or (RX_Length = 13) Then
                                RX_Index += 1
                            Else
                                RX_Index = 0
                            End If
                        Else
                            RX_Index += 1
                            If RX_Index >= RX_Length Then
                                RX_Index = 0

                                Dim checksum As UShort = 0
                                Dim n As Byte
                                For n = 0 To (RX_Length - 2)
                                    checksum += RX_Packet(n)
                                Next
                                If (checksum And 255) = RX_Packet(n) Then
                                    Packet_OK()                 ' 封包資料格式 OK
                                End If
                            End If
                        End If
                    End If
                End If
            Next
        End If
    End Sub

    Private Sub Packet_OK()                                     ' 封包資料格式 OK
        Select Case TX_Command
            Case CMD_ReadData
                If RX_Length = 13 Then
                    FG_Receive = True
                End If
            Case CMD_L1_Set
                If RX_Length = 5 Then
                    FG_Receive = True
                End If
            Case CMD_L2_Set
                If RX_Length = 5 Then
                    FG_Receive = True
                End If
            Case CMD_L1_Output
                If RX_Length = 5 Then
                    FG_Receive = True
                End If
            Case CMD_L2_Output
                If RX_Length = 5 Then
                    FG_Receive = True
                End If
            Case CMD_L1_OnOff
                If RX_Length = 5 Then
                    FG_Receive = True
                End If
            Case CMD_L2_OnOff
                If RX_Length = 5 Then
                    FG_Receive = True
                End If
            Case CMD_L1_TrigOut
                If RX_Length = 5 Then
                    FG_Receive = True
                End If
            Case CMD_L2_TrigOut
                If RX_Length = 5 Then
                    FG_Receive = True
                End If
        End Select
    End Sub

    ' L1輸出調整 ***************************************************************

    Private Sub cbb_L1_MaxC_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbb_L1_MaxC.SelectedIndexChanged
        If ((L1_Set >> 4) And &HF) <> cbb_L1_MaxC.SelectedIndex Then
            Dim i As Byte
            i = L1_Set And (SET_ExtOnOff Or SET_OnOff)
            L1_Set = (cbb_L1_MaxC.SelectedIndex << 4) Or i

            nud_L1_Output.Value = 0
            tkb_L1_Output.Value = 0
            L1_Output = 0
            CHG_L1_Output = False

            Dim MaxC As Integer = MaxCurrent(cbb_L1_MaxC.SelectedIndex)
            lb_L1_MaxC.Text = Convert.ToString(MaxC)
            tkb_L1_Output.TickFrequency = MaxC / 10
            nud_L1_Output.Maximum = MaxC
            tkb_L1_Output.Maximum = MaxC

            If FG_ReadData = True Then
                CHG_L1_Set = True
            End If
        End If
    End Sub

    Private Sub tkb_L1_Output_ValueChanged(sender As Object, e As EventArgs) Handles tkb_L1_Output.ValueChanged
        nud_L1_Output.Value = tkb_L1_Output.Value
        L1_Output = tkb_L1_Output.Value
        lb_L1_PCV.Text = String.Format("{0:F1}%", (L1_Output * 100) / MaxCurrent(cbb_L1_MaxC.SelectedIndex))

        If FG_ReadData = True Then
            CHG_L1_Output = True
        End If
    End Sub

    Private Sub nud_L1_Output_ValueChanged(sender As Object, e As EventArgs) Handles nud_L1_Output.ValueChanged
        tkb_L1_Output.Value = nud_L1_Output.Value
        L1_Output = tkb_L1_Output.Value
        lb_L1_PCV.Text = String.Format("{0:F1}%", (L1_Output * 100) / MaxCurrent(cbb_L1_MaxC.SelectedIndex))

        If FG_ReadData = True Then
            CHG_L1_Output = True
        End If
    End Sub

    ' L1 ON/OFF ================================================================

    Private Sub ckb_L1_ExtOnOff_CheckedChanged(sender As Object, e As EventArgs) Handles ckb_L1_ExtOnOff.CheckedChanged
        If ckb_L1_ExtOnOff.Checked = False Then                 ' Enable -> Disable ?
            L1_Set = L1_Set And (Not SET_ExtOnOff)              ' L1 EXT. ON/OFF = Disable
            lb_L1_LED.Visible = True
            btn_L1_OnOff.Visible = True
        Else                                                    ' Disable -> Enable
            L1_Set = L1_Set Or SET_ExtOnOff                     ' L1 EXT. ON/OFF = Enable
            lb_L1_LED.Visible = False
            btn_L1_OnOff.Visible = False
        End If

        If FG_ReadData = True Then
            CHG_L1_Set = True
        End If
    End Sub

    Private Sub btn_L1_OnOff_Click(sender As Object, e As EventArgs) Handles btn_L1_OnOff.Click
        If (L1_Set And SET_OnOff) = 0 Then                      ' OFF -> ON ?
            L1_Set = L1_Set Or SET_OnOff                        ' L1 ON
            lb_L1_LED.BackColor = Color.Green
            btn_L1_OnOff.Text = "OFF"
        Else                                                    ' ON -> OFF
            L1_Set = L1_Set And (Not SET_OnOff)                 ' L1 OFF
            lb_L1_LED.BackColor = Color.DimGray
            btn_L1_OnOff.Text = "ON"
        End If

        If FG_ReadData = True Then
            CHG_L1_OnOff = True
        End If
    End Sub

    ' L1觸發輸出延遲 ===========================================================

    Private Sub nud_L1_TrigOut_ValueChanged(sender As Object, e As EventArgs) Handles nud_L1_TrigOut.ValueChanged
        L1_TrigOut = nud_L1_TrigOut.Value
        lb_L1_TrigOut.Text = Convert.ToString(L1_TrigOut * 5) + "ms"

        If FG_ReadData = True Then
            CHG_L1_TrigOut = True
        End If
    End Sub

    ' L2輸出調整 ***************************************************************

    Private Sub cbb_L2_MaxC_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbb_L2_MaxC.SelectedIndexChanged
        If ((L2_Set >> 4) And &HF) <> cbb_L2_MaxC.SelectedIndex Then
            Dim i As Byte
            i = L2_Set And (SET_ExtOnOff Or SET_OnOff)
            L2_Set = (cbb_L2_MaxC.SelectedIndex << 4) Or i

            nud_L2_Output.Value = 0
            tkb_L2_Output.Value = 0
            L2_Output = 0
            CHG_L2_Output = False

            Dim MaxC As Integer = MaxCurrent(cbb_L2_MaxC.SelectedIndex)
            lb_L2_MaxC.Text = Convert.ToString(MaxC)
            tkb_L2_Output.TickFrequency = MaxC / 10
            nud_L2_Output.Maximum = MaxC
            tkb_L2_Output.Maximum = MaxC

            If FG_ReadData = True Then
                CHG_L2_Set = True
            End If
        End If
    End Sub

    Private Sub tkb_L2_Output_ValueChanged(sender As Object, e As EventArgs) Handles tkb_L2_Output.ValueChanged
        nud_L2_Output.Value = tkb_L2_Output.Value
        L2_Output = tkb_L2_Output.Value
        lb_L2_PCV.Text = String.Format("{0:F1}%", (L2_Output * 100) / MaxCurrent(cbb_L2_MaxC.SelectedIndex))

        If FG_ReadData = True Then
            CHG_L2_Output = True
        End If
    End Sub

    Private Sub nud_L2_Output_ValueChanged(sender As Object, e As EventArgs) Handles nud_L2_Output.ValueChanged
        tkb_L2_Output.Value = nud_L2_Output.Value
        L2_Output = tkb_L2_Output.Value
        lb_L2_PCV.Text = String.Format("{0:F1}%", (L2_Output * 100) / MaxCurrent(cbb_L2_MaxC.SelectedIndex))

        If FG_ReadData = True Then
            CHG_L2_Output = True
        End If
    End Sub

    ' L2 ON/OFF ================================================================

    Private Sub ckb_L2_ExtOnOff_CheckedChanged(sender As Object, e As EventArgs) Handles ckb_L2_ExtOnOff.CheckedChanged
        If ckb_L2_ExtOnOff.Checked = False Then                 ' Enable -> Disable ?
            L2_Set = L2_Set And (Not SET_ExtOnOff)              ' L2 EXT. ON/OFF = Disable
            lb_L2_LED.Visible = True
            btn_L2_OnOff.Visible = True
        Else                                                    ' Disable -> Enable
            L2_Set = L2_Set Or SET_ExtOnOff                     ' L2 EXT. ON/OFF = Enable
            lb_L2_LED.Visible = False
            btn_L2_OnOff.Visible = False
        End If

        If FG_ReadData = True Then
            CHG_L2_Set = True
        End If
    End Sub

    Private Sub btn_L2_OnOff_Click(sender As Object, e As EventArgs) Handles btn_L2_OnOff.Click
        If (L2_Set And SET_OnOff) = 0 Then                      ' OFF -> ON ?
            L2_Set = L2_Set Or SET_OnOff                        ' L2 ON
            lb_L2_LED.BackColor = Color.Green
            btn_L2_OnOff.Text = "OFF"
        Else                                                    ' ON -> OFF
            L2_Set = L2_Set And (Not SET_OnOff)                 ' L2 OFF
            lb_L2_LED.BackColor = Color.DimGray
            btn_L2_OnOff.Text = "ON"
        End If

        If FG_ReadData = True Then
            CHG_L2_OnOff = True
        End If
    End Sub

    Private Sub nud_L2_TrigOut_ValueChanged(sender As Object, e As EventArgs) Handles nud_L2_TrigOut.ValueChanged
        L2_TrigOut = nud_L2_TrigOut.Value
        lb_L2_TrigOut.Text = Convert.ToString(L2_TrigOut * 5) + "ms"

        If FG_ReadData = True Then
            CHG_L2_TrigOut = True
        End If
    End Sub
End Class
